;;;; package.lisp

(in-package :cl-user)

(defpackage :clevr-grammar
  (:documentation "FCG grammar for CLEVR dataset with IRL primitives")
  (:use :common-lisp :utils :fcg :clevr-world :clevr-primitives)
  (:import-from :monitors
                :activate-monitor)
  (:import-from :irl :bind
                :get-target-var)
  (:import-from :cl-json
                :decode-json-from-source)
  (:shadowing-import-from :fcg :size :attributes)
  (:export :*CLEVR*
           :AN-EQUAL-NUMBER-CXN :ARE-OTHER-CXN :ARE-THERE-ANY-X-CXN
           :ARE-THERE-ANY-X-OF-CXN :ARE-THERE-ANY-X-THAT-ARE-CXN
           :ARE-THERE-ANY-X-THAT-HAVE-CXN :BALL-MORPH-CXN
           :BALLS-MORPH-CXN :BASE-NOMINAL-CXN :BASE-RELATE-CXN
           :BASE-RELATE-THAT-IS-CXN :BEHIND-LEX-CXN :BEHIND-MORPH-CXN
           :BIG-MORPH-CXN :BLOCK-MORPH-CXN :BLOCKS-MORPH-CXN :BLUE-LEX-CXN
           :BOTH-RX-AND-RY-CXN :BROWN-LEX-CXN :COLOR-LEX-CXN
           :COMPARE-ANAPHORIC-DOES-CXN :COMPARE-ANAPHORIC-IS-CXN
           :COMPARE-ANAPHORIC-IS-SPLIT-CXN :COMPARE-ARE-MATERIAL-CXN
           :COMPARE-DO-CXN :COMPARE-DOES-CXN :COMPARE-EQUAL-COUNTABLE-CXN
           :COMPARE-IS-CXN :COMPARE-IS-MATERIAL-CXN :COMPARE-IS-SPLIT-CXN
           :COMPARE-UNCOUNTABLE-CXN :COMPARE-UNEQUAL-COUNTABLE-CXN
           :COUNT-OTHER-CXN :COUNT-ZERO-HOP-CXN :CUBE-LEX-CXN :CUBE-MORPH-CXN
           :CUBES-MORPH-CXN :CYAN-LEX-CXN :CYLINDER-LEX-CXN :CYLINDER-MORPH-CXN
           :CYLINDERS-MORPH-CXN :EITHER-X-OR-Y-CXN :EXIST-ZERO-HOP-CXN
           :EXIST-ZERO-HOP-VISIBLE-CXN :FEWER-CXN :FRONT-LEX-CXN :GRAY-LEX-CXN
           :GREATER-THAN-CXN :GREEN-LEX-CXN :HAS-WHAT-T-CXN :HOP-COUNT-ANAPHORIC-CXN
           :HOP-COUNT-CXN :HOP-EXIST-ANAPHORIC-CXN :HOP-EXIST-CXN :HOP-HOW-BIG-IS-IT-CXN
           :HOP-HOW-BIG-IS-X-CXN :HOP-QUERY-PROPERTY-ANAPHORIC-CXN :HOP-QUERY-PROPERTY-CXN
           :HOP-QUERY-PROPERTY-REVERSE-CXN :HOP-WHAT-IS-IT-MADE-OF-CXN
           :HOP-WHAT-IS-X-MADE-OF-CXN :HOP-WHAT-MATERIAL-IS-X-MADE-OF-CXN
           :HOP-X-IS-MADE-OF-WHAT-CXN :HOW-MANY-X-ARE-CXN :HOW-MANY-X-ARE-THERE-OF-CXN
           :HOW-MANY-X-HAVE-CXN :IN-FRONT-OF-MORPH-CXN :INTERSECT-COUNT-CXN
           :INTERSECT-HOW-BIG-ANAPHORIC-CXN :INTERSECT-HOW-BIG-CXN
           :INTERSECT-MADE-OF-ANAPHORIC-CXN :INTERSECT-MADE-OF-ANAPHORIC-EXTENDED-CXN
           :INTERSECT-MADE-OF-CXN :INTERSECT-MADE-OF-REVERSE-CXN
           :INTERSECT-QUERY-PROPERTY-ANAPHORIC-CXN :INTERSECT-QUERY-PROPERTY-CXN
           :INTERSECT-QUERY-PROPERTY-REVERSE-CXN :IS-THERE-A-X-CXN
           :IS-THERE-A-X-OF-CXN :IS-THERE-A-X-THAT-HAS-CXN :IS-THERE-A-X-THAT-IS-CXN
           :IS-THERE-ANOTHER-X-CXN :IS-THERE-ANOTHER-X-OF-CXN :IS-THERE-ANOTHER-X-THAT-HAS-CXN
           :IS-THERE-ANOTHER-X-THAT-IS-CXN :IS-THERE-ANY-OTHER-X-OF-CXN
           :IS-THERE-ANY-OTHER-X-THAT-HAS-CXN :IS-THERE-ANY-OTHER-X-THAT-IS-CXN
           :IS-THERE-ANYTHING-ELSE-OF-CXN :IS-THERE-ANYTHING-ELSE-THAT-HAS-CXN
           :IS-THERE-ANYTHING-ELSE-THAT-IS-CXN :IS-WHAT-T-CXN :LARGE-LEX-CXN
           :LARGE-MORPH-CXN :LEFT-LEX-CXN :LEFT-OF-MORPH-CXN :LESS-THAN-CXN
           :MATERIAL-LEX-CXN :MATTE-MORPH-CXN :METAL-LEX-CXN :METAL-MORPH-CXN
           :METALLIC-MORPH-CXN :MORE-CXN :NOMINAL-CXN :OBJECT-MORPH-CXN
           :OBJECTS-MORPH-CXN :ON-THE-LEFT-SIDE-OF-MORPH-CXN
           :ON-THE-RIGHT-SIDE-OF-MORPH-CXN :PLURAL-RELATE-CXN
           :PLURAL-RELATE-THAT-ARE-CXN :PURPLE-LEX-CXN :RED-LEX-CXN
           :RELATE-CXN :RELATE-THAT-IS-CXN :RIGHT-LEX-CXN :RIGHT-OF-MORPH-CXN
           :RUBBER-LEX-CXN :RUBBER-MORPH-CXN :RX-AND-RY-CXN :SAME-RELATE-COUNT-CXN
           :SAME-RELATE-COUNT-MATERIAL-CXN :SAME-RELATE-EXIST-CXN
           :SAME-RELATE-EXIST-MATERIAL-CXN :SAME-RELATE-QUERY-ANAPHORIC-ANOTHER-CXN
           :SAME-RELATE-QUERY-ANAPHORIC-ANOTHER-MADE-OF-CXN
           :SAME-RELATE-QUERY-ANAPHORIC-ANOTHER-MATERIAL-CXN
           :SAME-RELATE-QUERY-ANAPHORIC-CXN
           :SAME-RELATE-QUERY-ANAPHORIC-MADE-OF-CXN
           :SAME-RELATE-QUERY-ANAPHORIC-MATERIAL-CXN :SAME-RELATE-QUERY-CXN
           :SAME-RELATE-QUERY-MATERIAL-CXN :SAME-RELATE-QUERY-OTHER-CXN
           :SAME-RELATE-QUERY-OTHER-MATERIAL-CXN :SAME-RELATE-QUERY-REVERSE-CXN
           :SAME-RELATE-QUERY-REVERSE-MATERIAL-CXN :SAME-RELATE-QUERY-REVERSE-OTHER-CXN
           :SAME-RELATE-QUERY-REVERSE-OTHER-MATERIAL-CXN :SHAPE-LEX-CXN
           :SHINY-MORPH-CXN :SIZE-LEX-CXN :SMALL-LEX-CXN :SMALL-MORPH-CXN
           :SPHERE-LEX-CXN :SPHERE-MORPH-CXN :SPHERES-MORPH-CXN
           :T-THE-SAME-AS-CXN :THE-SAME-AS-CXN :THE-SAME-NUMBER-CXN
           :THE-SAME-T-AS-COMPARE-CXN :THE-SAME-T-AS-RELATE-CXN
           :THE-SAME-T-CXN :THE-T-OF-THE-SAME-AS-CXN :THING-LEX-CXN
           :THING-MORPH-CXN :THINGS-MORPH-CXN :TINY-MORPH-CXN
           :TO-THE-LEFT-OF-MORPH-CXN :TO-THE-RIGHT-OF-MORPH-CXN
           :UNION-COUNT-CXN :UNIQUE-DECLARED-CXN :UNIQUE-DETERMINED-CXN
           :WHAT-IS-ITS-T-CXN :WHAT-IS-THE-T-OF-CXN :WHAT-NUMBER-OF-X-ARE-CXN
           :WHAT-NUMBER-OF-X-ARE-THERE-OF-CXN :WHAT-NUMBER-OF-X-HAVE-CXN
           :WHAT-T-IS-CXN :WHAT-T-IS-IT-CXN :X-OR-Y-CXN :YELLOW-LEX-CXN
           :clevr-meaning->rpn :rpn->irl))
